@@ -9,10 +9,9 @@ import typing as t
 import uuid
 from datetime import datetime, timedelta
 from pathlib import PurePath
-from urllib.parse import urljoin
+from urllib.parse import urlencode, urljoin
 
 import httpx
-from google.auth.transport.requests import Request as GoogleRequest  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
 from .encoders import aps_encoder
@@ -27,8 +26,7 @@ from .messages import (
     Notification,
     PushNotification,
 )
-from .utils import cleanup_firebase_message, make_async
-
+from .utils import cleanup_firebase_message
 
 DEFAULT_TTL = 604800
 
@@ -41,6 +39,7 @@ class AsyncFirebaseClient:
     """
 
     BASE_URL: str = "https://fcm.googleapis.com"
+    TOKEN_URL: str = "https://oauth2.googleapis.com/token"
     FCM_ENDPOINT: str = "/v1/projects/{project_id}/messages:send"
     # A list of accessible OAuth 2.0 scopes can be found https://developers.google.com/identity/protocols/oauth2/scopes.
     SCOPES: t.List[str] = [
@@ -66,8 +65,8 @@ class AsyncFirebaseClient:
 
         :param scopes: user-defined scopes to request during the authorization grant.
         """
-        self._credentials = credentials
-        self.scopes = scopes or self.SCOPES
+        self._credentials: service_account.Credentials = credentials
+        self.scopes: t.List[str] = scopes or self.SCOPES
 
     def creds_from_service_account_info(self, service_account_info: t.Dict[str, str]) -> None:
         """Creates a Credentials instance from parsed service account info.
@@ -91,20 +90,25 @@ class AsyncFirebaseClient:
             filename=service_account_filename, scopes=self.scopes
         )
 
-    @make_async
-    def _get_access_token(self) -> t.Coroutine[t.Any, t.Any, t.Any]:
-        """Retrieve a valid access token that can be used to authorize requests.
-        :return: Access token
-        """
-        if self._credentials.valid:  # type: ignore
-            return self._credentials.token  # type: ignore
+    async def _get_access_token(self) -> str:
+        if self._credentials.valid:
+            return self._credentials.token
 
-        request = GoogleRequest()
-        self._credentials.refresh(request)  # type: ignore
-        logging.debug(
-            "Obtained access token %s that can be used to authorize requests.", self._credentials.token  # type: ignore
-        )
-        return self._credentials.token  # type: ignore
+        headers = {"Content-Type": "application/x-www-form-urlencoded"}
+        data = urlencode(
+            {
+                "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
+                "assertion": self._credentials._make_authorization_grant_assertion(),
+            }
+        ).encode("utf-8")
+
+        async with httpx.AsyncClient() as client:
+            response: httpx.Response = await client.post(self.TOKEN_URL, data=data, headers=headers)
+            response_data = response.json()
+
+        self._credentials.expiry = datetime.utcnow() + timedelta(seconds=response_data["expires_in"])
+        self._credentials.token = response_data["access_token"]
+        return self._credentials.token
 
     @staticmethod
     def build_android_config(  # pylint: disable=too-many-locals
@@ -394,6 +398,10 @@ class AsyncFirebaseClient:
             )
             logging.debug("Requesting POST %s, payload: %s", url, payload)
             response: httpx.Response = await client.post(url, json=payload, headers=await self._prepare_headers())
-            logging.debug("Response Code: %s, Time spent to make a request: %s", response.status_code, response.elapsed)
+            logging.debug(
+                "Response Code: %s, Time spent to make a request: %s",
+                response.status_code,
+                response.elapsed,
+            )
 
         return response
